<?php

namespace Drupal\gtext\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;

use Drupal\Component\Gettext\PoHeader;
use Drupal\Component\Gettext\PoItem;
use Drupal\Component\Gettext\PoStreamWriter;
use Drupal\Core\Ajax\AppendCommand;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Url;

use Drupal\gtext\GoogleTranslate;

class APIController {
    public function deleteTranslate($lid){
        \Drupal::service('language_manager')->reset();

        $response  = new AjaxResponse();

        \Drupal::database()->delete('locales_target')->condition('lid', $lid)->execute();
        \Drupal::database()->delete('locales_source')->condition('lid', $lid)->execute();

        $languages = \Drupal::languageManager()->getLanguages();
        _locale_refresh_translations(array_keys($languages), [$lid]);
        _locale_refresh_configuration(array_keys($languages), [$lid]);

        $response->addCommand(new InvokeCommand('body', 'append', ['<script>location.reload();</script>']));

        \Drupal::service('language_manager')->reset();
        return $response;
    }
    public function googleTranslateText(){
        $response  = new AjaxResponse();
        if ( !empty($_POST['target']) && !empty($_POST['lang']) && !empty($_POST['text']) ){
            $translate = GoogleTranslate::translate(!empty($_POST['source']) ? $_POST['source'] : 'auto', $_POST['lang'], $_POST['text']);

            if( !empty($translate) ){
                $translate = str_replace(trim($_POST['text']), $translate, $_POST['text']);
                $response->addCommand(new InvokeCommand('#'.$_POST['target'], 'val', [$translate]));
            }
        }
        $response->addCommand(new InvokeCommand('#'.$_POST['target'], 'removeClass', ['gtext-in-process']));
        $response->addCommand(new InvokeCommand('#'.$_POST['target'], 'removeClass', ['gtext-in-process']));
        $response->addCommand(new AppendCommand('body', ['#type' => 'status_messages']));
        return $response;
    }
    public function translateExport($langcode, $group){
        $filename = str_replace('www.', '', $_SERVER['HTTP_HOST']) . '-' . ($group == '_all' ? 'all' : $group). '-'.$langcode.'-'.date('Y-m-d_His').'.po';

        $strings = $this->getStrings($langcode, $group);
        if ( !empty($strings) ){
            $uri = \Drupal::service('file_system')->getTempDirectory().'/'.$filename;

            $header  = new PoHeader($langcode);
            $header->setProjectName(str_replace('www.', '', $_SERVER['HTTP_HOST']) . '-' . ($group == '_all' ? 'all' : $group));
            $header->setLanguageName($langcode);
            $header->setFromString('Plural-Forms: '.$this->getPluralForm($langcode));

            $writer = new PoStreamWriter();
            $writer->setHeader($header);
            $writer->setURI($uri);
            $writer->open();
            foreach($strings as $string){
                $text = (array) $string;
                if ( empty($string->translation) ){
                    continue;
                }
                $item = new PoItem();
                $item->setFromArray($text);
                $writer->writeItem($item);
            }
            $writer->close();

            $response = new BinaryFileResponse($uri, 200, [], FALSE, 'attachment');
            $response->deleteFileAfterSend(TRUE);
            return $response;
        } else {
            \Drupal::messenger()->addMessage(gtext()->gtext('Not found strings for export'));
            return new RedirectResponse(Url::fromRoute('gtext.translate'));

        }
    }
    protected function getStrings($langcode, $context = '_all'){
        $conditions = [
            'language' => $langcode,
        ];
        $options    = [
            'translated'     => TRUE,
        ];

        if ( $context != '_none' ){
            if ( empty($context) || $context == '_all' ){
                $contexts = array_keys(\Drupal::moduleHandler()->invokeAll('gtext_contexts'));
                if ( empty($contexts) ){
                    $contexts[] = 'not_exists';
                }
                $conditions['context'] = $contexts;
            } else {
                $options['filters']['context'] = $context;
            }
        }

        return \Drupal::service('locale.storage')->getTranslations($conditions, $options);
    }

    /**
     * @see http://docs.translatehouse.org/projects/localization-guide/en/latest/l10n/pluralforms.html?id=l10n/pluralforms
     */
    protected function getPluralForm($langcode){
        switch($langcode){
            case 'ru':
            case 'uk':
            case 'be':
                return 'nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);';
            default:
                return 'nplurals=2; plural=(n != 1);';
        }
    }
}
