<?php

namespace Drupal\gtext\Controller;

use \Drupal\gtext\Form\TranslateForm;
use \Drupal\gtext\Form\TranslateFilterForm;
use Drupal\gtext\Form\TranslateStringForm;
use Drupal\locale\SourceString;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class TranslateController {
    public function translatePage(){
        $filter_form    = \Drupal::formBuilder()->getForm(TranslateFilterForm::class);
        $translate_form = \Drupal::formBuilder()->getForm(TranslateForm::class);
        return [
            'filter'    => $filter_form,
            'translate' =>  $translate_form,
        ];
    }
    public function translateStringPage($lid){
        if ( \Drupal::request()->query->get('action', 'form') == 'close' ){
            unset($_SESSION['gtext'][$lid]);
            return [
                '#type'  => 'html_tag',
                '#tag'   => 'script',
                '#value' => 'opener.reloadTranslations('.$lid.'); window.close();',
            ];
        }
        $string = \Drupal::service('locale.storage')->findTranslation(['lid' => $lid]);
        if ( empty($string) ){
            throw new NotFoundHttpException();
        }
        return \Drupal::formBuilder()->getForm(TranslateStringForm::class, $string);
    }
    public function translateStringTitle($lid){
        $string = \Drupal::service('locale.storage')->findTranslation(['lid' => $lid, 'language' => 'en']);
        if ( empty($string) ){
            throw new NotFoundHttpException();
        }
        $source            = new SourceString($string);
        $source_array      = $source->getPlurals();
        return gtext()->gtext("Translating \"%string\"", [
            '%string' => $source_array[0]
        ]);
    }
    public function translateStringUpdate($lid){
        $result = [];
        foreach(\Drupal::languageManager()->getLanguages() as $langcode => $language){
            if ( !\locale_is_translatable($langcode)) {
                continue;
            }

            $string = \Drupal::service('locale.storage')->getTranslations([
                'lid'        => $lid,
                'language'   => $langcode,
                'translated' => TRUE
            ]);

            $source_array = [];
            if ( $string ){
                $string = reset($string);
                $source_array = $string->getPlurals();
            }
            if ( empty($source_array) ){
                for($i = 0; $i <=2; $i++){
                    $result['strings['.$lid.']['.$langcode.']['.$i.']'] = '';
                }
            } else {
                foreach($source_array as $i => $str){
                    $result['strings['.$lid.']['.$langcode.']['.$i.']'] = $str;
                }
            }
        }
        return new JsonResponse($result);
    }
}

