<?php
namespace Drupal\gtext\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

class TranslateFilterForm extends FormBase {
    public function getFormId(){
        return 'gtext_translate_filter';
    }
    public function buildForm(array $form, FormStateInterface $form_state){
        \Drupal::service('language_manager')->reset();

        $form['#attributes']['class'][]   = 'glyanec-translate-form-filter';

        $allLanguages = \Drupal::languageManager()->getLanguages();
        $languages    = [];
        $ntrans       = [];
        foreach($allLanguages as $langcode => $language){
            if ( $langcode == 'en' ) {
                continue;
            }
            $languages[$langcode] = $language;
            $ntrans['_ntrans:'.$langcode] = gtext()->gtext('All not translated strings in %lang', ['%lang' => $language->getName()]);
        }

        $status = self::getAPIStatus();
        $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

        // filter entity.configurable_language.collection
        $form['filter']  = [
            '#type'         => 'details',
            '#title'        => gtext()->gtext->t('Filter translatable strings') . \Drupal::service('renderer')->render($status),
            '#open'         => TRUE,
            'string' => [
                '#type'         => 'search',
                '#title'        => gtext()->gtext->t('String contains'),
                '#description'  => gtext()->gtext->t('Leave blank to show all strings. The search is case sensitive.'),
                '#default_value'=> isset($_GET['string']) ? $_GET['string'] : '',
            ],
            'context' => [
                '#type'         => 'select',
                '#title'        => gtext()->gtext->t('Text group'),
                '#options'      => [
                    'Filters'  => [
                        '_all'    => gtext()->gtext('All contexts'),
                        '_none'   => gtext()->gtext('All translatable strings'),
                    ] + $ntrans,
                    'Contexts' => \Drupal::moduleHandler()->invokeAll('gtext_contexts'),
                ],
                '#default_value'=> isset($_GET['context']) ? $_GET['context'] : '_none',
            ],
            'actions' => [
                '#type'         => 'actions',
                '#attributes'   => ['class' => ['container-inline']],
                'submit' => [
                    '#type'         => 'submit',
                    '#value'        => gtext()->gtext->t('Filter'),
                ],
                'reset' => [
                    '#markup'       => Link::fromTextAndUrl(
                                            gtext()->gtext->t('Reset'),
                                            new Url('gtext.translate', [], ['attributes' => ['class' => ['button']]])
                                        )->toString(),
                ],
            ],
        ];
        return $form;
    }
    public function submitForm(array &$form, FormStateInterface $form_state){
        $values = $form_state->getValues();
        $query  = [];
        if ( !empty($values['string']) ){
            $query['string'] = $values['string'];
        }
        if ( !empty($values['context']) ){
            $query['context'] = $values['context'];
        }
        $url = Url::fromRoute('gtext.translate', [], ['query' => $query]);
        $form_state->setRedirectUrl($url);
    }

    public static function getAPIStatus(){
        $apiKey = \Drupal::config('gtext.settings')->get('google_api_key', false);
        if ( empty($apiKey) ){
            $keyStatus   = gtext()->gtext('not set');
            $statusClass = 'not-set';
        } else {
            $keyStatus   = $apiKey;
            $statusClass = 'set';
        }
        $status = [
            '#type'       => 'html_tag',
            '#tag'        => 'div',
            '#value'      => gtext()->gtext('Google API key: @key', ['@key' => $keyStatus]),
            '#attributes' => [
                'id'        => 'google-api-status',
                'class'     => ['google-api-status', $statusClass]
            ],
            'link'        => [
                '#type'     => 'link',
                '#url'      => Url::fromRoute('gtext.translate.settings'),
                '#title'    => gtext()->gtext('Settings'),
                '#attributes' => [
                    'class'               => ['api-settings-link', 'use-ajax'],
                    'data-dialog-type'    => 'modal',
                    'data-dialog-options' => Json::encode([
                        'width' => '90%',
                    ]),
                ],
            ]
        ];
        return $status;
    }
}

