<?php
namespace Drupal\gtext\Form;

use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Render\Markup;
use Drupal\Component\Gettext\PoStreamWriter;
use Drupal\Core\Database\Query\Condition;

use Drupal\locale\SourceString;
use Drupal\Component\Gettext\PoItem;


class TranslateForm extends FormBase {
    public function getFormId(){
        return 'gtext_translate';
    }
    public function buildForm(array $form, FormStateInterface $form_state){
        \Drupal::service('language_manager')->reset();
        $form['#attached'] = [
            'library'           => [
                'gtext/gtext',
            ]
        ];

        $form['#attributes']['class'][]   = 'glyanec-translate-form';

        $allLanguages = \Drupal::languageManager()->getLanguages();
        $languages    = [];
        $ntrans       = [];
        foreach($allLanguages as $langcode => $language){
            if ( $langcode == 'en' ) {
                continue;
            }
            $languages[$langcode] = $language;
            $ntrans['_ntrans:'.$langcode] = gtext()->gtext('All not translated strings in %lang', ['%lang' => $language->getName()]);
        }

        $translate_english = \locale_is_translatable('en');

        $strings      = $this->getStrings($languages, isset($_GET['string']) ? $_GET['string'] : false, isset($_GET['context']) ? $_GET['context'] : '_none');
        $pluralCount  = [];

        foreach($languages as $langcode => $language){
            $pluralCount[$langcode] = $this->getNumberOfPlurals($langcode);
            $header[$langcode]      = [
                'data'          => $language->getName(),
                'data-col-lang' => $langcode,
            ];
        }

        if( empty($strings['en']) ){
            $form['strings'] = array(
                '#type'   => 'table',
                '#header' => array_merge([gtext()->gtext->t('Source string')], $header),
                '#rows'   => [],
                '#empty'  => gtext()->gtext->t('Not found strings'),
                '#sticky' => true,
            );
            return $form;
        }

        $form['strings'] = array(
            '#type'   => 'table',
            '#sticky' => true,
            '#header' => array_merge([$this->t('Source string')], $header),
            '#attributes' => [
                'class' => ['translate-form-table']
            ]
        );

        $form['strings']['#header']['save'] = [
            'class' => ['fixed-save-button-wrapper'],
            'data'  => Markup::create('<div class="fixed-save-button"></div>'),
        ];
        foreach($strings['en'] as $string){
            $source            = new SourceString($string);
            $source_array      = $source->getPlurals();

            foreach($languages as $langcode => $language){
                $translation_array[$langcode] = $strings[$langcode][$string->lid]->getPlurals();
            }

            if ( count($source_array) == 1 ){
                $plural = FALSE;
                $form['strings'][$string->lid]['original'] = [
                    '#type'          => 'item',
                    '#title'         => $this->t('Source string (@language)', ['@language' => $this->t('Built-in English')]),
                    '#title_display' => 'invisible',
                    '#plain_text'    => $source_array[0],
                    '#weight'        => 0,
                ];
                if( $translate_english && !empty($string->customized) ){
                    $form['strings'][$string->lid]['original']['#markup'] = '<s title="'.gtext()->gtext('Original string').'">'.$form['strings'][$string->lid]['original']['#plain_text'].'</s><br><span class="gtext-replaced-string-'.$string->lid.'" title="'.gtext()->gtext('Replaced string').'"><i>'.$string->translation.'</i></span>';
                    unset($form['strings'][$string->lid]['original']['#plain_text']);
                }
                foreach($languages as $langcode => $language){
                    $form['strings'][$string->lid][$langcode][0] = [
                        '#type'           => 'textarea',
                        '#title'          => $this->t('Translated string (@language)', ['@language' => $langcode]),
                        '#title_display'  => 'invisible',
                        '#rows'           => 2,
                        '#cols'           => 40,
                        '#default_value'  => $translation_array[$langcode][0],
                        '#attributes'     => [
                            'lang'      => $langcode,
                            'class'     => ['gtext-translatable-field'],
                            'data-lang' => $langcode,
                            'data-lid'  => $string->lid,
                            'data-text' => $source_array[0],
                            'data-url'  => Url::fromRoute('gtext.translate.google')->toString()
                        ],
                    ];
                }
            } else {
                $plural = TRUE;
                $original_singular = [
                    '#type'       => 'item',
                    '#title'      => $this->t('Singular form'),
                    '#plain_text' => $source_array[0],
                    '#prefix'     => '<span class="visually-hidden">' . $this->t('Source string (@language)', ['@language' => $this->t('Built-in English')]) . '</span><span lang="en">',
                    '#suffix'     => '</span>',
                ];
                $original_plural = [
                    '#type'       => 'item',
                    '#title'      => $this->t('Plural form'),
                    '#plain_text' => $source_array[1],
                    '#preffix'    => '<span lang="en">',
                    '#suffix'     => '</span>',
                ];
                if( $translate_english && !empty($string->customized) ){
                    if ( $original_singular['#plain_text'] != $string->getPlurals()[0] ){
                        $original_singular['#markup'] = '<s title="'.gtext()->gtext('Original string').'">'.$original_singular['#plain_text'].'</s><br><span class="gtext-replaced-string-'.$string->lid.'" title="'.gtext()->gtext('Replaced string').'"><i>'.$string->getPlurals()[0].'</i></span>';
                        unset($original_singular['#plain_text']);
                    }

                    if ( $original_plural['#plain_text'] != $string->getPlurals()[1] ){
                        $original_plural['#markup'] = '<s title="'.gtext()->gtext('Original string').'">'.$original_plural['#plain_text'].'</s><br><span class="gtext-replaced-string-'.$string->lid.'" title="'.gtext()->gtext('Replaced string').'"><i>'.$string->getPlurals()[0].'</i></span>';
                        unset($original_plural['#plain_text']);
                    }
                }
                $form['strings'][$string->lid]['original'] = [
                    'original_singular' => $original_singular + ['#weight' => 0],
                    'original_plural'   => $original_plural   + ['#weight' => 2],
                ];
                foreach($languages as $langcode => $language){
                    for ($i = 0; $i < $pluralCount[$langcode]; $i++) {
                        $form['strings'][$string->lid][$langcode][$i] = [
                            '#type'          => 'textarea',
                            '#title'         => ($i == 0 ? $this->t('Singular form') : $this->formatPlural($i, 'First plural form', '@count. plural form')),
                            '#rows'          => 2,
                            '#cols'          => 40,
                            '#default_value' => isset($translation_array[$langcode][$i]) ? $translation_array[$langcode][$i] : '',
                            '#attributes'    => [
                                'lang'           => $langcode,
                                'class'          => ['gtext-translatable-field'],
                                'data-lang'      => $langcode,
                                'data-text'      => $source_array[$i == 0 ? 0 : 1],
                                'data-url'       => Url::fromRoute('gtext.translate.google')->toString()
                            ],
                            '#prefix'        => $i == 0 ? ('<span class="visually-hidden">' . $this->t('Translated string (@language)', ['@language' => $langcode]) . '</span>') : '',
                            '#weight'        => $i * 2,
                        ];
                    }
                    if ($pluralCount[$langcode] == 2) {
                        $form['strings'][$string->lid][$langcode][1]['#title'] = $this->t('Plural form');
                    }
                }
            }

            $form['strings'][$string->lid]['save'] = [];


            $form['strings'][$string->lid]['original']['delete'] = [
                '#weight'   => -10,
                '#markup'   => Link::fromTextAndUrl(
                                        '',
                                        Url::fromUserInput('#delete', [
                                            'attributes' => [
                                                'class'    => ['gtext-remove-button'],
                                                'title'    => gtext()->gtext->t('Delete'),
                                                'data-url' => Url::fromRoute('gtext.translate.delete', ['lid' => $string->lid])->toString(),
                                            ]
                                        ])
                            )->toString(),
            ];
            $form['strings'][$string->lid]['original']['edit_form'] = [
                '#weight'  => -10,
                '#markup'  => Markup::create('<a href="javascript:void(0)" class="gtext-edit-form-link" title="'.gtext()->gtext->t('Edit translations on new tab').'" data-url="'.Url::fromRoute('gtext.translate.string', ['lid' => $string->lid])->toString().'"><svg xmlns="http://www.w3.org/2000/svg" width="16px" viewBox="0 0 448 512"><path fill="#00f" d="M448 80v352c0 26.51-21.49 48-48 48H48c-26.51 0-48-21.49-48-48V80c0-26.51 21.49-48 48-48h352c26.51 0 48 21.49 48 48zm-88 16H248.029c-21.313 0-32.08 25.861-16.971 40.971l31.984 31.987L67.515 364.485c-4.686 4.686-4.686 12.284 0 16.971l31.029 31.029c4.687 4.686 12.285 4.686 16.971 0l195.526-195.526 31.988 31.991C358.058 263.977 384 253.425 384 231.979V120c0-13.255-10.745-24-24-24z"/></svg></a>')
            ];
            if ( $translate_english ){
                $form['strings'][$string->lid]['original']['edit'] = [
                    '#weight'  => -9,
                    '#markup' => Link::fromTextAndUrl(
                                            '',
                                            Url::fromUserInput('#replace', [
                                                'attributes' => [
                                                    'data-lid' => $string->lid,
                                                    'class'    => ['gtext-edit-button'],
                                                    'title'    => gtext()->gtext->t('Replace')
                                                ]
                                            ])
                                )->toString(),
                ];
                if ( $plural ){
                    for ($i = 0; $i < $pluralCount[$langcode]; $i++) {
                        $form['strings'][$string->lid]['original'][$i] = [
                            '#type'             => 'textarea',
                            '#rows'             => 2,
                            '#cols'             => 40,
                            '#default_value'    => !empty($string->customized) ? $string->getPlurals()[$i] : '',
                            '#parents'          => ['strings', $string->lid, 'en', $i],
                            '#attributes'       => [
                                'class' => ['replace_en_'.$string->lid],
                                'style' => 'display:none',
                            ],
                            '#weight'        => $i * 2 + 1,
                        ];
                    }
                    if ($pluralCount[$langcode] == 2) {
                        $form['strings'][$string->lid][$langcode][1]['#title'] = $this->t('Plural form');
                    }
                } else {
                    $form['strings'][$string->lid]['original'][0] = [
                        '#type'             => 'textarea',
                        '#rows'             => 2,
                        '#cols'             => 40,
                        '#default_value'    => !empty($string->customized) ? $string->translation : '',
                        '#parents'          => ['strings', $string->lid, 'en', 0],
                        '#attributes'       => [
                            'class' => ['replace_en_'.$string->lid],
                            'style' => 'display:none',
                        ],
                        '#weight'        => 1,
                    ];
                }

            }
            if (!empty($string->context)) {
                $form['strings'][$string->lid]['original'][] = [
                    '#type'         => 'inline_template',
                    '#template'     => '<br><small>{{ context_title }}: <span lang="en">{{ context }}</span></small>',
                    '#context'      => [
                        'context_title' => $this->t('In Context'),
                        'context'       => $string->context,
                    ],
                    '#weight'        => 10,
                ];
            }
        }
        krsort($form['strings']);

        $form['actions'] = [
            '#type'         => 'actions',
        ];
        $form['actions']['submit'] = [
            '#type'         => 'submit',
            '#value'        => $this->t('Save translations'),
            '#submit'       => ['::saveTranslate'],
            '#id'           => 'save_strings',
        ];

        $form['pager']['#type'] = 'pager';


        $links = [];
        foreach ($languages as $langcode => $language){
            $links[] = [
                'title'     => gtext()->gtext->t('Export %lang', ['%lang' => $language->getName()]),
                'url'       => new Url('gtext.translate.export', [
                                                    'langcode' => $language->getId(),
                                                    'group'    => isset($_GET['context']) ? $_GET['context'] : '_all'
                            ])
            ];
        }
        $form['actions']['export'] = [
            '#type'         => 'dropbutton',
            '#links'        => $links,
            '#prefix'       => Markup::create('<div style="float:right;">'),
            '#suffix'       => '</div>'
        ];
        if ( isset($_GET['context']) && strpos($_GET['context'], '_ntrans:') !== false ){
            unset($form['actions']['export']);
        }
        return $form;
    }
    public function submitForm(array &$form, FormStateInterface $form_state){
    }
    public function formFilter(array &$form, FormStateInterface $form_state){
        $form_state->setRedirect('gtext.translate', [], [
            'query'     => [
                'string'  => $form_state->getValue('string'),
                'context' =>$form_state->getValue('context'),
            ]
        ]);
    }
    public function formReset(array &$form, FormStateInterface $form_state){
        $form_state->setRedirect('gtext.translate');
    }
    public function saveTranslate(array &$form, FormStateInterface $form_state){
        $updated           = [];

        $translate_english = \locale_is_translatable('en');

        $languages         = [];
        $existing_objects  = [];
        $lids              = array_keys($form_state->getValue('strings'));

        $allLanguages      = \Drupal::languageManager()->getLanguages();
        foreach($allLanguages as $langcode => $language){
            if ( !$translate_english && $langcode == 'en' ) {
                continue;
            }
            $languages[$langcode] = $language;
            foreach (\Drupal::service('locale.storage')->getTranslations(['lid' => $lids, 'language' => $langcode, 'translated' => TRUE]) as $existing_translation_object) {
                $existing_objects[$existing_translation_object->lid][$langcode] = $existing_translation_object;
            }
        }

        foreach ($form_state->getValue('strings') as $lid => $new_translation) {
            foreach($languages as $langcode => $language){
                // $new_translation[$langcode]       = array_map('trim', $new_translation[$langcode]);
                $new_translation_string_delimited = implode(PoItem::DELIMITER, $new_translation[$langcode]);

                $existing_translation             = isset($existing_objects[$lid][$langcode]);
                $new_translation_string           = implode('', $new_translation[$langcode]);

                $is_changed = FALSE;
                if ( $existing_translation && $existing_objects[$lid][$langcode]->translation != $new_translation_string_delimited ) {
                    $is_changed = TRUE;
                } else
                if ( !$existing_translation && !empty($new_translation_string) ) {
                    $is_changed = TRUE;
                }

                if ( $is_changed ) {
                    $target = null;
                    if ( isset($existing_objects[$lid][$langcode]) ){
                        $target = $existing_objects[$lid][$langcode];
                    } else {
                        $target = \Drupal::service('locale.storage')->createTranslation(['lid' => $lid, 'language' => $langcode]);
                    }
                    $target->setPlurals($new_translation[$langcode])
                        ->setCustomized()
                        ->save();
                    $updated[] = $target->getId();
                }
                if (empty($new_translation_string) && isset($existing_objects[$lid][$langcode])) {
                    $existing_objects[$lid][$langcode]->delete();
                    $updated[] = $lid;
                }
            }
        }

        $this->messenger()->addStatus($this->t('The strings have been saved.'));

        if ($updated) {
            _locale_refresh_translations([$langcode], $updated);
            _locale_refresh_configuration([$langcode], $updated);
        }
    }

    private function getStrings($languages, $string = false, $context = false){
        $strings   = [];
        $lids      = [];

        $query = \Drupal::database()
            ->select('locales_source', 's')
            ->extend('Drupal\\Core\\Database\\Query\\PagerSelectExtender');
        $query->fields('s', ['lid']);

        foreach ($languages as $langcode => $language){
            $query->leftJoin('locales_target', $langcode, $langcode.'.lid = s.lid AND '.$langcode.'.language = \''.$langcode.'\'');
            $query->addExpression($langcode.'.translation', $langcode);
        }
        // filter
        if(!empty($string)){
            $db_or = new Condition('OR');
            $db_or->condition('s.source', '%'.trim($string).'%', 'LIKE');
            foreach ($languages as $langcode => $language){
                $db_or->condition($langcode.'.translation', '%'.trim($string).'%', 'LIKE');
            }
            $query->condition($db_or);
        }

        if ( strpos($context, '_ntrans:') !== false ){ // Не перекладені на мову
            $lng = substr($context, 8);
            $query->condition($lng.'.translation', null, 'IS NULL');
        } else
        if ( $context != '_none'  ){ // Всі контексти
            if ( empty($context) || $context == '_all' ){ // Всі існуючі контексти
                $contexts = array_keys($this->getAllContexts());
                if ( empty($contexts) ){
                    $contexts[] = 'not_exists';
                }
                $conditions['context'] = $contexts;
                $query->condition('s.context', $contexts, 'IN');
            } else {                                     // Конкретний контекст
                $query->condition('s.context', $context);
            }
        }
        $query->limit(30);
        $query->orderBy('s.lid', 'DESC');
        $lids = $query->execute()->fetchCol();

        // Load languages
        $languages['en'] = true;
        foreach($languages as $langcode => $language){
            if ( !empty($lids) ){
                $conditions = [
                    'language' => $langcode,
                    'lid'      => $lids,
                ];
                $stringsLang = \Drupal::service('locale.storage')->getTranslations($conditions, []);
                foreach($stringsLang as $string){
                    $strings[$langcode][$string->lid] = $string;
                }
            } else {
                $strings[$langcode] = [];
            }
        }

        return $strings;
    }

    private function getAllContexts(){
        return \Drupal::moduleHandler()->invokeAll('gtext_contexts');
    }
}
