<?php
namespace Drupal\gtext;

use Drupal\Core\Url;
use Google\Cloud\Translate\V2\TranslateClient;
use Exception;

class GoogleTranslate {
    public static function translate($source, $target, $text) {
        $translation = false;

        $apiKey = \Drupal::config('gtext.settings')->get('google_api_key', false);
        if ( !empty($apiKey) ){
            try {
                $client = new TranslateClient([
                    'key' => $apiKey,
                ]);
                $response = $client->translate($text, [
                    'target' => $target,
                ]);
                if ( !empty($response['text']) ){
                    $translation = $response['text'];
                }
            } catch(Exception $e){
                $message = @json_decode($e->getMessage(), true);
                if ( $message !== false && isset($message['error']['code']) && isset($message['error']['message']) ){
                    $message = '['.$message['error']['code'].'] ' . $message['error']['message'];
                } else {
                    $message = $e->getMessage();
                }
                \Drupal::messenger('gtext')->addWarning($message);
            }
        }
        if ( $translation === false ){
            try {
            $response    = self::requestTranslation($source, $target, $text);
            $translation = self::getSentencesFromJSON($response);
            } catch(Exception $e){
                \Drupal::messenger('gtext')->addWarning($e->getMessage());
                return false;
            }
        }
        return $translation;
    }
    protected static function requestTranslation($source, $target, $text){
        if(mb_strlen($text) >= 1000){
            throw new \Exception( gtext()->gtext(
                'Maximum number of characters exceeded: 1000. To increase the limits, <a href="@url">enter the Google Translate key on the page</a>', [
                    '@url' => Url::fromRoute('gtext.translate.settings_page')->toString(),
                ]
            ).'');
        }

        $url = "https://translate.google.com/translate_a/single?client=at&dt=t&dt=ld&dt=qca&dt=rm&dt=bd&dj=1&hl=es-ES&ie=UTF-8&oe=UTF-8&inputm=2&otf=2&iid=1dd3b944-fa62-4b55-b330-74909a99969e";
        $fields = array(
            'sl' => urlencode($source),
            'tl' => urlencode($target),
            'q' => urlencode($text)
        );
        $fields_string = "";
        foreach ($fields as $key => $value) {
            $fields_string .= $key . '=' . $value . '&';
        }
        rtrim($fields_string, '&');
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, count($fields));
        curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_ENCODING, 'UTF-8');
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_USERAGENT, 'AndroidTranslate/5.3.0.RC02.130475354-53000263 5.1 phone TRANSLATE_OPM5_TEST_1');
        $result = curl_exec($ch);
        curl_close($ch);
        return $result;
    }
    protected static function getSentencesFromJSON($json){
        $sentencesArray = json_decode($json, true);
        $sentences = "";
        foreach ($sentencesArray["sentences"] as $s) {
            $sentences .= isset($s["trans"]) ? $s["trans"] : '';
        }
        return $sentences;
    }
}
